@@ -1,7 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003-2004 Imendio HB
- * Copyright (C) 2003-2004 Sjoerd Simons <sjoerd@luon.net>
+ * Copyright (C) 2003-2005 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License as
@@ -77,6 +76,7 @@ struct _LmConnection {
 	guint         io_watch_err;
 	guint         io_watch_hup;
 	guint         fd;
+	guint         io_watch_connect;
 	
 	guint         open_id;
 	LmCallback   *open_cb;
@@ -183,7 +183,8 @@ static GSourceFuncs incoming_funcs = {
 static void
 connection_free (LmConnection *connection)
 {
-	int i;
+	int        i;
+	LmMessage *m;
 
 	g_free (connection->server);
 	g_free (connection->jid);
@@ -203,11 +204,19 @@ connection_free (LmConnection *connection)
 	}
 
 	g_hash_table_destroy (connection->id_handlers);
-
-	if (lm_connection_is_open (connection)) {
+	if (connection->state >= LM_CONNECTION_STATE_OPENING) {
 		connection_do_close (connection);
 	}
 
+	while ((m = g_queue_pop_head (connection->incoming_messages)) != NULL) {
+		lm_message_unref (m);
+	}
+
+	if (connection->ssl) {
+		lm_ssl_unref (connection->ssl);
+	}
+
+	g_queue_free (connection->incoming_messages);
 	g_free (connection);
 }
 
@@ -258,11 +267,18 @@ connection_new_message_cb (LmParser     *parser,
 			   LmMessage    *m,
 			   LmConnection *connection)
 {
+	const gchar *from;
+	
 	lm_message_ref (m);
 
+	from = lm_message_node_get_attribute (m->node, "from");
+	if (!from) {
+		from = "unknown";
+	}
+	
 	lm_verbose ("New message with type=\"%s\" from: %s\n",
 		    _lm_message_type_to_string (lm_message_get_type (m)),
-		    lm_message_node_get_attribute (m->node, "from"));
+		    from);
 
 	g_queue_push_tail (connection->incoming_messages, m);
 }
@@ -277,6 +293,12 @@ _lm_connection_succeeded (LmConnectData *connect_data)
 	gchar        *ch;
 
 	connection = connect_data->connection;
+	
+	if (connection->io_watch_connect != 0) {
+		g_source_destroy (g_main_context_find_source_by_id(connection->context,
+								   connection->io_watch_connect));
+		connection->io_watch_connect = 0;
+	}
 
 	/* Need some way to report error/success */
 	if (connection->cancel_open) {
@@ -306,9 +328,8 @@ _lm_connection_succeeded (LmConnectData *connect_data)
 				    NULL)) {
 			shutdown (connection->fd, SHUT_RDWR);
 			close (connection->fd);
+		
 			connection_do_close (connection);
-			connection->fd = -1;
-			g_io_channel_unref(connection->io_channel);
 			return FALSE;
 		}
 	}
@@ -377,17 +398,35 @@ _lm_connection_succeeded (LmConnectData *connect_data)
 void 
 _lm_connection_failed_with_error (LmConnectData *connect_data, int error) 
 {
+	LmConnection *connection;
+	
 	g_log (LM_LOG_DOMAIN, LM_LOG_LEVEL_NET,
 	       "Connection failed: %s (error %d)\n",
 	       strerror (error), error);
 	
+	connection = connect_data->connection;
+	
 	connect_data->current_addr = connect_data->current_addr->ai_next;
 	
+	if (connection->io_watch_connect != 0) {
+		g_source_destroy (g_main_context_find_source_by_id(connection->context,
+								   connection->io_watch_connect));
+		connection->io_watch_connect = 0;
+	}
+
 	if (connect_data->io_channel != NULL) {
 		g_io_channel_unref (connect_data->io_channel);
 	}
 	
 	if (connect_data->current_addr == NULL) {
+		connection->state = LM_CONNECTION_STATE_CLOSED;
+		if (connection->open_cb && connection->open_cb->func) {
+			LmCallback *cb = connection->open_cb;
+			
+			(* ((LmResultFunction) cb->func)) (connection, FALSE,
+							   cb->user_data);
+		}
+		
 		freeaddrinfo (connect_data->resolved_addrs);
 		g_free (connect_data);
 	} else {
@@ -407,13 +446,11 @@ connection_connect_cb (GIOChannel   *source,
 		       GIOCondition  condition,
 		       gpointer      data) 
 {
-	LmConnection  *connection;
 	LmConnectData *connect_data;
 	int            error;
-	int            len  = sizeof(error);
+	guint          len  = sizeof(error);
 
 	connect_data = (LmConnectData *) data;
-	connection   = connect_data->connection;
 	
 	if (condition == G_IO_ERR) {
 		/* get the real error from the socket */
@@ -485,12 +522,14 @@ connection_do_connect (LmConnectData *connect_data)
 	
 	connect_data->io_channel = g_io_channel_unix_new (fd);
 	if (connection->proxy) {
+		connection->io_watch_connect =
 		connection_add_watch (connection,
 				      connect_data->io_channel,
 				      G_IO_OUT|G_IO_ERR,
 				      (GIOFunc) _lm_proxy_connect_cb, 
 				      connect_data);
 	} else {
+		connection->io_watch_connect =
 		connection_add_watch (connection,
 				      connect_data->io_channel,
 				      G_IO_OUT|G_IO_ERR,
@@ -518,6 +557,7 @@ connection_add_watch (LmConnection *connection,
 	g_source_set_callback (source, (GSourceFunc)func, user_data, NULL);
                                                                                 
 	id = g_source_attach (source, connection->context);
+
 	g_source_unref (source);
   
 	return id;
@@ -540,9 +580,12 @@ connection_start_keep_alive (LmConnection *connection)
 		connection_stop_keep_alive (connection);
 	}
 
-	connection->keep_alive_id = g_timeout_add (connection->keep_alive_rate,
-						   (GSourceFunc) connection_send_keep_alive,
-						   connection);
+	if (connection->keep_alive_rate > 0) {
+		connection->keep_alive_id =
+			g_timeout_add (connection->keep_alive_rate,
+				       (GSourceFunc) connection_send_keep_alive,
+				       connection);
+	}
 }
 
 static void
@@ -649,15 +692,27 @@ connection_do_close (LmConnection *connection)
 	connection_stop_keep_alive (connection);
 
 	if (connection->io_channel) {
-		g_source_remove (connection->io_watch_in);
-		g_source_remove (connection->io_watch_err);
-		g_source_remove (connection->io_watch_hup);
+		g_source_destroy (g_main_context_find_source_by_id (
+			connection->context, connection->io_watch_in));
+		g_source_destroy (g_main_context_find_source_by_id (
+			connection->context, connection->io_watch_err));
+		g_source_destroy (g_main_context_find_source_by_id (
+			connection->context, connection->io_watch_hup));
+
+		if (connection->io_watch_connect != 0) {
+			g_source_destroy (g_main_context_find_source_by_id(connection->context,
+									   connection->io_watch_connect));
+			connection->io_watch_connect = 0;
+		}
 
 		g_io_channel_unref (connection->io_channel);
 		connection->io_channel = NULL;
+
+		connection->fd = -1;
 	}
 
-	g_source_remove (g_source_get_id (connection->incoming_source));
+	
+	g_source_destroy (connection->incoming_source);
 	g_source_unref (connection->incoming_source);
 
 	if (!lm_connection_is_open (connection)) {
@@ -678,7 +733,7 @@ connection_in_event (GIOChannel   *source,
 		     LmConnection *connection)
 {
 	gchar     buf[IN_BUFFER_SIZE];
-	gint      bytes_read;
+	gsize     bytes_read;
 	GIOStatus status;
        
 	if (!connection->io_channel) {
@@ -695,7 +750,7 @@ connection_in_event (GIOChannel   *source,
 						  NULL);
 	}
 
-	if (status != G_IO_STATUS_NORMAL) {
+	if (status != G_IO_STATUS_NORMAL || bytes_read < 0) {
 		gint reason;
 		
 		switch (status) {
@@ -720,14 +775,14 @@ connection_in_event (GIOChannel   *source,
 
 	buf[bytes_read] = '\0';
 	g_log (LM_LOG_DOMAIN, LM_LOG_LEVEL_NET, "\nRECV [%d]:\n", 
-	       bytes_read);
+	       (int)bytes_read);
 	g_log (LM_LOG_DOMAIN, LM_LOG_LEVEL_NET, 
 	       "-----------------------------------\n");
 	g_log (LM_LOG_DOMAIN, LM_LOG_LEVEL_NET, "'%s'\n", buf);
 	g_log (LM_LOG_DOMAIN, LM_LOG_LEVEL_NET, 
 	       "-----------------------------------\n");
 
-	lm_verbose ("Read: %d chars\n", bytes_read);
+	lm_verbose ("Read: %d chars\n", (int)bytes_read);
 
 	lm_parser_parse (connection->parser, buf);
 	
@@ -1118,6 +1173,7 @@ lm_connection_new (const gchar *server)
 	connection->cancel_open       = FALSE;
 	connection->state             = LM_CONNECTION_STATE_CLOSED;
 	connection->keep_alive_id     = 0;
+	connection->keep_alive_rate   = 0;
 	
 	connection->id_handlers = g_hash_table_new_full (g_str_hash, 
 							 g_str_equal,
@@ -1382,11 +1438,14 @@ lm_connection_authenticate_and_block (LmConnection  *connection,
 		return FALSE;
 	}
 
+	connection->state = LM_CONNECTION_STATE_AUTHENTICATING;
+
 	m = connection_create_auth_req_msg (username);
 	result = lm_connection_send_with_reply_and_block (connection, m, error);
 	lm_message_unref (m);
 
 	if (!result) {
+		connection->state = LM_CONNECTION_STATE_OPEN;
 		return FALSE;
 	}
 
@@ -1399,6 +1458,7 @@ lm_connection_authenticate_and_block (LmConnection  *connection,
 
 	result = lm_connection_send_with_reply_and_block (connection, m, error);
 	if (!result) {
+		connection->state = LM_CONNECTION_STATE_OPEN;
 		return FALSE;
 	}
 
@@ -1407,9 +1467,11 @@ lm_connection_authenticate_and_block (LmConnection  *connection,
 	
 	switch (type) {
 	case LM_MESSAGE_SUB_TYPE_RESULT:
+		connection->state = LM_CONNECTION_STATE_AUTHENTICATED;
 		return TRUE;
 		break;
 	case LM_MESSAGE_SUB_TYPE_ERROR:
+		connection->state = LM_CONNECTION_STATE_OPEN;
 		g_set_error (error,
 			     LM_ERROR,
 			     LM_ERROR_AUTH_FAILED,
@@ -1622,6 +1684,7 @@ void
 lm_connection_set_ssl (LmConnection *connection, LmSSL *ssl)
 {
 	g_return_if_fail (connection != NULL);
+	g_return_if_fail (lm_ssl_is_supported () == TRUE);
 
 	if (connection->ssl) {
 		lm_ssl_unref (connection->ssl);
@@ -1655,15 +1718,13 @@ lm_connection_get_proxy (LmConnection *connection)
  * @connection: an #LmConnection
  * @proxy: an #LmProxy
  *
- * Sets the proxy to use for this connection. To unset pass a proxy of type 
- * #LM_PROXY_TYPE_NONE to this function.
+ * Sets the proxy to use for this connection. To unset pass #NULL.
  * 
  **/
 void
 lm_connection_set_proxy (LmConnection *connection, LmProxy *proxy)
 {
 	g_return_if_fail (connection != NULL);
-	g_return_if_fail (proxy != NULL);
 
 	if (lm_connection_is_open (connection)) {
 		g_warning ("Can't change server proxy while connected");
@@ -1675,7 +1736,7 @@ lm_connection_set_proxy (LmConnection *connection, LmProxy *proxy)
 		connection->proxy = NULL;
 	}
 
-	if (lm_proxy_get_type (proxy) != LM_PROXY_TYPE_NONE) {
+	if (proxy && lm_proxy_get_type (proxy) != LM_PROXY_TYPE_NONE) {
 		connection->proxy = lm_proxy_ref (proxy);
 	}
 }
@@ -1771,6 +1832,15 @@ lm_connection_send_with_reply_and_block (LmConnection  *connection,
 	g_return_val_if_fail (connection != NULL, NULL);
 	g_return_val_if_fail (message != NULL, NULL);
 
+	if (connection->state < LM_CONNECTION_STATE_OPENING) {
+		g_set_error (error,
+			     LM_ERROR,
+			     LM_ERROR_CONNECTION_NOT_OPEN,
+			     "Connection is not open, call lm_connection_open() first");
+		return FALSE;
+	}
+
+
 	if (lm_message_node_get_attribute (message->node, "id")) {
 		id = g_strdup (lm_message_node_get_attribute (message->node, 
 							      "id"));
