@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003 Imendio HB
+ * Copyright (C) 2003 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License as
@@ -27,6 +27,8 @@
 
 #include <glib.h>
 
+G_BEGIN_DECLS
+
 #define LM_ERROR lm_error_quark ()
 
 typedef enum {
@@ -38,4 +40,6 @@ typedef enum {
 
 GQuark lm_error_quark (void) G_GNUC_CONST;
 
+G_END_DECLS
+
 #endif /* __LM_ERROR_H__ */
